using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using UMD.HCIL.Piccolo;
using UMD.HCIL.Piccolo.Nodes;

namespace Piccolo_1D_Puzzle {
	public class Puzzle : System.Windows.Forms.Form {
		private int numTiles = 8;       // Number of non-blank tiles
		private int blankTileIndex;     // Zero-based position of blank tile
		private PCanvas canvas;
		private System.ComponentModel.Container components = null;

		public Puzzle() {
			InitializeComponent();

			// Create the Piccolo canvas
			blankTileIndex = numTiles;
			canvas = new PCanvas();
			canvas.Bounds = this.ClientRectangle;
			canvas.Anchor = AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top | AnchorStyles.Bottom;
			canvas.Camera.MouseDown += new PInputEventHandler(Camera_MouseDown);
			Controls.Add(canvas);

			// Create the scenegraph after Piccolo loads
			Load += new EventHandler(Puzzle_Load);
		}

		#region Dispose method
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// Puzzle
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(744, 134);
			this.Name = "Puzzle";
			this.Text = "Form1";

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new Puzzle());
		}

		private void Puzzle_Load(object sender, EventArgs e) {
			ResetPuzzle();
		}

		private void Camera_MouseDown(object sender, UMD.HCIL.Piccolo.Event.PInputEventArgs e) {
			PTile tile = e.PickedNode as PTile;
			if (tile != null) {
				if (!tile.Blank) {
					GameLogicPress(tile);
				}
			}
		}

		// Create Piccolo scenegraph for puzzle - one node per tile
		private void ResetPuzzle() {
			canvas.Layer.RemoveAllChildren();   // Remove any existing nodes 

			// Create set of tiles - one of which is blank.
			PTile tile;
			for (int i=0; i<=numTiles; i++) {
				tile = new PTile("ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i].ToString());
				canvas.Layer.AddChild(tile);
				tile.TilePosition = i;
				if (i == blankTileIndex) {
					tile.Blank = true;
				}
			}
		}

		// Perform the game action when a tile is clicked on (assumes tile is not Blank).
		// This shifts all the tiles between the clicked tile (inclusive) and the blank tile towards the blank tile.
		// Then the blank tile is moved to the position that the clicked on tile was at.
		private void GameLogicPress(PTile clickTile) {
			int clickTileIndex = clickTile.TilePosition;

			// Then, animate each tile to its new spot
			PTile blankTile = (PTile)canvas.Layer.GetChild(blankTileIndex);
			if (clickTileIndex < blankTileIndex) {
				for (int i=clickTileIndex; i<=(blankTileIndex-1); i++) {
					PTile tile = (PTile)canvas.Layer.GetChild(i);
					tile.TilePosition++;
				}
				blankTile.MoveInBackOf(clickTile);
			} else {
				for (int i=(blankTileIndex+1); i<=clickTileIndex; i++) {
					PTile tile = (PTile)canvas.Layer.GetChild(i);
					tile.TilePosition--;
				}
				blankTile.MoveInFrontOf(clickTile);
			}
			blankTileIndex = clickTileIndex;
			blankTile.TilePosition = blankTileIndex;
		}
	}
}